
use strict;
use warnings;
use Carp;
use Getopt::Long qw(:config no_ignore_case );
use File::Find;
use Data::Dumper;

BEGIN {
    push @INC, ".";
    push @INC, "sbin";
    require "getlib.conf";
}

use vars qw( $LIB_NAME $LIB_URL $LIB_UNPACK $LIB_VERSION $LIB_FILE
	     $LIB_DIR $debug $help $opts $LIB_ACTION_DO $CONF $LIB_WORKDIR
	     $SCRIPT $get $found $path @dirs @match $PERL_VAR $OS $custom );

$SCRIPT = "getlib";

#====================================
#    - - -    S E T U P   - - -
#====================================

# get command line options, if problems, set help flag
$help = 1  unless ( &GetOptions(  "name=s"    => \$opts->{'name'},
                                  "verbose+"  => \$opts->{'verbose'},
                                  "debug!"    => \$opts->{'debug'},
                                  "help!"     => \$opts->{'help'},
                                )
                   );

my $limit = "===================\n";

croak "$SCRIPT: error, library name argument missing.\n"
    unless $opts->{name};

unless ($CONF->{$opts->{name}}) { 
    my $str = "$limit $SCRIPT: ERROR, library \"".$opts->{name}."\" doesn't exist " .
        "in getlib.conf.\n$limit Available options are:\n";
    while ( (my $key, my $val) = each %$CONF) {
        $str .= " + $key\n";
    }
    croak $str;
}

$LIB_NAME       = $CONF->{$opts->{name}}{name};
$LIB_URL        = $CONF->{$opts->{name}}{url};
$LIB_UNPACK     = $CONF->{$opts->{name}}{unpack};
$LIB_VERSION    = $CONF->{$opts->{name}}{version};
$LIB_ACTION_DO  = $CONF->{$opts->{name}}{download_only};
$LIB_WORKDIR    = $CONF->{$opts->{name}}{workdir} || "src"; 

# List of mandatory binaries
my @binaries_mandatory = ("tar","bunzip2"); 

# check mandatory binaries
if (&check_binaries(@binaries_mandatory) != 0) {
   croak &tellf("some required binary files are missing") .
         &tellfe("Libraries not downladed and upacked") .
         &tellfe("Your install is likely to fail.") .
         &tellfe("Install missing binaries and rerun.");
}

@dirs      = qw( /usr/bin /usr/local/bin );
@match     = qw( wget curl lwp-download GET );

#===============================
#    - - -    R U N   - - -
#===============================

find ( \&wanted, @dirs );

croak &tellfe("No downloadig utilities found") .
    &tellfe("install at least one of: " . join(", ",@match) ) 
    unless $found;

my @parts = split("/",$LIB_URL);
my $file  = $parts[$#parts];

#=========================================
# !!! WE OPERATE IN WORKING DIRECTORY !!!
#=========================================
if (!-d $LIB_WORKDIR) {
	mkdir $LIB_WORKDIR;
}
chdir $LIB_WORKDIR;

foreach (@match) {
        /curl/         && do { $get = $path->{$_} . " -O $LIB_URL"      ; last };
        /wget/         && do { $get = $path->{$_} . " $LIB_URL"         ; last };
        /lwp-download/ && do { $get = $path->{$_} . " $LIB_URL"         ; last };
        /GET/          && do { $get = $path->{$_} . " $LIB_URL > $file" ; last };
}

$PERL_VAR = substr($],0,5);
$OS = qx/uname -a | cut -d" " -f1/;
$OS =~ s/\s//;

$custom->{perl}    = $LIB_NAME . "_perl" . $PERL_VAR  . ".pl";
$custom->{os}      = $LIB_NAME . "_" . $OS . ".pl";
$custom->{perl_os} = $LIB_NAME . "_" . $PERL_VAR . "_" . $OS . ".pl";

if ($opts->{debug}) {
    printf "$SCRIPT: %-4s = %6s \n", "Perl",$PERL_VAR;
    printf "$SCRIPT: %-4s = %6s \n", "OS", $OS;
}

my @tmp = split("/",$LIB_URL);
$LIB_FILE = $tmp[$#tmp];
@tmp = split(".tar",$LIB_FILE);
$LIB_DIR = $tmp[0];

if (-e $LIB_FILE) {
    &tell("FOUND existing file $LIB_FILE");
} else {
    &tell("DOWNLOADING $LIB_NAME with '$get'\n");
    system($get) == 0 or die "$SCRIPT: ERROR couldn't download source ($get)";
}

&process unless $LIB_ACTION_DO;

exit 0;

#=========================================
#    - - -  SUBROUTINES  - - - 
#=========================================
sub wanted {
    foreach my $file (@match) {
        if ($file eq $_) {
            push @$found, $file;
            $path->{$file} = $File::Find::name;
        }
    }
}

sub process {

    my $unpack    = "$LIB_UNPACK $file";
    my $bzip_file = $file;
    $bzip_file =~ s/.bz2//;

    if ( $LIB_UNPACK =~ /tar/ && -d $LIB_DIR) {

	&tell("FOUND existing directory $LIB_DIR");

    } elsif ( $LIB_UNPACK =~ /bunzip2/ && -f $bzip_file) {

	&tell("FOUND existing unpacked file $bzip_file");

    } else {

	&tell("UNPACKING $LIB_NAME with '$unpack'");
	system($unpack) == 0 or die "$SCRIPT: ERROR $unpack";
	
	my @customize;
	foreach (keys %$custom) {
	    push @customize, $_ if -e "_custom/" . $custom->{$_};
	}
	
	foreach (@customize) {
	    my $script = "$LIB_WORKDIR/_custom/" . $custom->{$_};
	    &tell("CUSTOMIZING $LIB_NAME with '$script'");
	    my $cmd = "_custom/" . $custom->{$_}; 
	    system($cmd) == 0 or die "$SCRIPT: ERROR $script";
	}
    }
}

sub tell {
    my $text  = shift;
    my $fmode = shift;

    # return string mode [1] 
    return sprintf ("%s", "$SCRIPT: [$LIB_NAME] $text \n")
        if $fmode && $fmode == 1;    

    # return string mode [2], with error
    return sprintf ("%s", "$SCRIPT: [$LIB_NAME] ERROR !!! $text !!! \n")
        if $fmode && $fmode == 2;    

    # ordinary mode, print only
    print "$SCRIPT: [$LIB_NAME] $text \n";
}

# convinience aliases
sub tellf  { my $t = shift; return &tell($t,1); }
sub tellfe { my $t = shift; return &tell($t,2); }

sub check_binaries {

   my $unchecked = 0;
   foreach (@_) {
      print "$SCRIPT: checking $_... ";
      my $fail = system("which $_ &> /dev/null");
      if ($fail) {
         print " not found\n";
	 $unchecked++;
      }
      else {
         my $where = `which $_`;
         chomp($where);
         print "$where\n";
      }
   }
   return $unchecked; 

}
